/* TODO:
	- should cancel any previous dirlistings if a new listing is requested
	- remove cookie stuff
	
*/
#include "remoteview.h"

#include <storage/file.h>
#include <storage/path.h>
#include <util/message.h>

FTPRemoteView::FTPRemoteView( const Rect& cFrame, const String& zName ) : IconView( cFrame, zName, CF_FOLLOW_RIGHT | CF_FOLLOW_BOTTOM )
{
	m_pcServer = NULL;
	m_zPath = "";

	m_bUpdatePending = 0;
	m_nUpdateCookie = 0;
	
	m_pcDirChangedMsg = NULL;
	
	SetAutoSort( false );
	
#if 0
	/* TODO: this crashes when quitting !? */
//	SetFlags( GetFlags() | WID_DRAW_ON_CHILDREN );
	m_pcStringView = new StringView( GetBounds(), "remote_stringview", "", ALIGN_CENTER, CF_FOLLOW_ALL );
	AddChild( m_pcStringView );
	m_pcStringView->Hide();
#endif
}

FTPRemoteView::~FTPRemoteView()
{
	if( m_pcDirChangedMsg ) delete( m_pcDirChangedMsg );
#if 0
	RemoveChild( m_pcStringView );
	delete( m_pcStringView );
#endif
}

void FTPRemoteView::SetDirChangeMsg( Message* pcMsg )
{
	if( m_pcDirChangedMsg ) delete( m_pcDirChangedMsg );
	m_pcDirChangedMsg = pcMsg;
}

void FTPRemoteView::SetServer( Server* pcServer )
{
	/* TODO: Find out from the server what the current dir is via PWD */
	m_pcServer = pcServer;
	m_zPath = "/";
	Update();
}

Server* FTPRemoteView::GetServer()
{
	return( m_pcServer );
}

void FTPRemoteView::SetPath( const String& zPath )
{
	m_zPath = zPath;
	Update();
}

String FTPRemoteView::GetPath()
{
	return( m_zPath );
}

void FTPRemoteView::AttachedToWindow()
{
	Update();
}

#if 0
void FTPRemoteView::Paint( const Rect& cUpdateRect )
{
	if( m_pcServer == NULL ) {
		/* Draw a message if we aren't connected to a server */
		DrawMessage( "Not connected!\nEnter a server in the toolbar above and hit 'Connect'." );	/* TODO: localised strings */
	} else if( m_bUpdatePending && GetIconCount() == 0 ) {
		/* Draw a 'Waiting for server' message */
		DrawMessage( "Waiting for server..." );	/* TODO: localised strings */
	} else {
		IconView::Paint( cUpdateRect );
	}
}

void FTPRemoteView::DrawMessage( const String& zMsg )
{
	printf( "DrawMessage( %s )\n", zMsg.c_str() );
	EraseRect( GetBounds() );
	SetFgColor( Color32_s( 255,0,0 ) );
	Point cSize = GetTextExtent( zMsg );
	Point cPos;
	cPos.x = Width()/2 - cSize.x/2;
	cPos.y = Height()/2 - cSize.y/2;
	DrawString( cPos, zMsg );
	Flush();
}
#endif

void FTPRemoteView::ShowMessage( const String& zMsg )
{
	printf( "ShowMessage( %s )\n", zMsg.c_str() );
#if 0
	m_pcStringView->SetString( zMsg );
	if( !m_pcStringView->IsVisible() ) m_pcStringView->Show();
#endif
}

void FTPRemoteView::HideMessage()
{
	printf( "HideMessage()\n" );
#if 0
	m_pcStringView->SetString( "" );
	if( m_pcStringView->IsVisible() ) m_pcStringView->Hide();
#endif
}

/* Update: Request the ServerConnection to download a directory listing and send it to us asynchronously.
	When the listing is downloaded, it is sent back to us via a M_REMOTE_DIRLISTING message and SetContents() is called.
*/
void FTPRemoteView::Update()
{
	Clear();  /* Remove all icons */
	if( m_pcServer ) {
		m_bUpdatePending = true;
		m_nUpdateCookie++;
		m_pcServer->GetDirListing( m_zPath, this );
		ShowMessage( "Waiting for server..." );	/* TODO: localise strings */
	} else {
		ShowMessage( "Not connected.\nEnter a server in the toolbar above and hit 'Connect'." );
	}
}

/* GetNodeImage: Get a suitable icon for the file.
	If bSmall, icon should be 24x24; otherwise 48x48.
	TODO: Ask the registrar for a suitable icon, based on the file extension.
*/
Image* FTPRemoteView::GetNodeImage( RemoteNode* pcNode, bool bSmall )
{
	File* pcFile;
	/* Todo: ask the Registrar for suitable icon */
	/* Looks like with current RegistrarManager class, either have to parse the list of registered types, or create a 'fake' temp file and query registrar about that instead */
	if( pcNode->m_bIsDir ) {
		/* TODO: fall back to resource if file isn't present */
		pcFile = new File( "/system/icons/folder.png" );
	} else {
		pcFile = new File( "/system/icons/file.png" );
	}
	BitmapImage* pcImage = new BitmapImage( pcFile );
	
	if( bSmall ) {
		if( pcImage && pcImage->GetSize() != Point( 24,24 ) )
			pcImage->SetSize( Point( 24,24 ) );
	} else {
		if( pcImage && pcImage->GetSize() != Point( 48,48 ) )
			pcImage->SetSize( Point( 48,48 ) );
	}
	return( pcImage );
}

/* SetContents: Create & display icons for each of the nodes in pacNodes.
     If bInitial, this is the first of several messages and we should clear the iconview first.
     If bFinal, this is the last of the directory listing, and we should Layout() when done.
*/
void FTPRemoteView::SetContents( std::vector< RemoteNode >* pacNodes, bool bInitial, bool bFinal )
{
//	if( bInitial ) Clear();	/* Remove all icons */

	HideMessage();

	int nCount = pacNodes->size();
//	printf( "FTPRemoteView: Dirlisting fragment contains %i nodes; %s initial, %sfinal\n", nCount, bInitial?"":"not ", bFinal?"":"not " );

	for( int i = 0; i < nCount; i++ ) {
		RemoteNode cNode = (*pacNodes)[i];
//		printf( "  '%s', %s, size %i, permissions %i\n", cNode.GetName().c_str(), (cNode.IsDir() ? "dir" : "file"), cNode.GetSize(), cNode.GetPermissions() );

		Image* pcImage = GetNodeImage( &cNode, (GetView() == VIEW_LIST || GetView() == VIEW_DETAILS) );
		RemoteIconData* pcData = new RemoteIconData;
		pcData->m_cNode = cNode;

		uint nIndex = AddIcon( pcImage, pcData );
		AddIconString( nIndex, cNode.GetName() );

		String zTmp;
		if( cNode.IsDir() ) {
			zTmp = "<Dir>";
		} else {
			zTmp.Format( "%li", cNode.GetSize() );	/* TODO: human-readable (option) */
		}
		AddIconString( nIndex, zTmp );
		
		AddIconString( nIndex, cNode.GetTimestamp().GetDate() );
	}
	if( bFinal ) {
		Layout();
		m_bUpdatePending = false;
	}
	/* TODO: Perform periodic layouts while downloading large dirlisting */
}

void FTPRemoteView::HandleMessage( Message* pcMessage )
{
	switch( pcMessage->GetCode() )
	{
		case M_REMOTE_DIRLISTING:
		{
			if( !m_bUpdatePending ) {
				printf( "FTPRemoteView: Got REMOTE_DIRLISTING while no dirlisting is pending!\n" );
				return;
			}
			
			String zPath;
			if( pcMessage->FindString( "path", &zPath ) != 0 ) {
				printf( "FTPRemoteView: Got REMOTE_DIRLISTING without path!\n" );
				return;
			}
			if( zPath != m_zPath ) {
				printf( "FTPRemoteView: Got REMOTE_DIRLISTING with wrong path %s! Expecting %s.\n", zPath.c_str(), m_zPath.c_str() );
				return;
			}
			
			std::vector< RemoteNode >* pacNodes;
			/* The Server creates an array of RemoteNodes and passes us the pointer. We should delete it when done. */
			if( pcMessage->FindPointer( "list", (void**)&pacNodes ) ) {
				printf( "FTPRemoteView: Got REMOTE_DIRLISTING without pointer to data!\n" );
				return;
			}
			
			bool bInitial;
			if( pcMessage->FindBool( "initial", &bInitial ) != 0 ) bInitial = false;
			
			bool bFinal;
			if( pcMessage->FindBool( "final", &bFinal ) != 0 ) bFinal = false;
			
			SetContents( pacNodes, bInitial, bFinal );
			if( pacNodes ) delete( pacNodes );
		
			break;
		}
		default:
		{
			IconView::HandleMessage( pcMessage );
		}
	}
}

/* When a directory is invoked, change to it. TODO: What to do when a file is invoked? */
void FTPRemoteView::Invoked( uint nIcon, IconData* pcData )
{
//	printf( "FTPRemoteView::Invoked on icon %i (%s)\n", nIcon, GetIconString( nIcon, 0 ).c_str() );
	
	RemoteIconData* pcRData = (RemoteIconData*)pcData;
	
	if( pcRData->m_cNode.IsDir() ) {
		Path cPath = m_zPath;
		cPath.Append( pcRData->m_cNode.m_zName );
		printf( "New path: '%s'\n", cPath.GetPath().c_str() );
		printf( "RemoteView: Changing to %s\n", cPath.GetPath().c_str() );
		if( m_pcDirChangedMsg ) {
			Message cTmp = *m_pcDirChangedMsg;
			cTmp.AddString( "file/path", cPath.GetPath() );
			Invoke( &cTmp );
		}
		SetPath( cPath.GetPath() );
	}
}

/* MouseUp: We do the 'drop' part of drag & drop here. */
void FTPRemoteView::MouseUp( const Point& cPoint, uint32 nButtons, Message* pcData )
{
	if( pcData == NULL || m_pcServer == NULL ) {
		/* No drag & drop, or no connection */
		IconView::MouseUp( cPoint, nButtons, pcData );
		return;
	}
	StopScroll();
	
	/* We got a drag & drop. Check if it contains a local file */
	int nCount, nType;
	pcData->GetNameInfo( "file/path", &nType, &nCount );
	if( nType != T_STRING || nCount == 0 ) {
		/* Something was dropped that isn't a file. Ignore it */
		IconView::MouseUp( cPoint, nButtons, pcData );
		return;
	}
	
	/* Check if there is a directory icon under the mouse. If so, we transfer the file into that directory.
	   Otherwise, we transfer it into this directory. */
	String zDestPath = m_zPath;
	for( uint i = 0; i < GetIconCount(); i++ ) {
		if( Rect( GetIconPosition( i ), GetIconPosition( i ) + GetIconSize() ).DoIntersect( ConvertToView( cPoint ) ) ) {
			/* Found an icon under the mouse */
			RemoteIconData* pcData = (RemoteIconData*)GetIconData( i );
			if( pcData->m_cNode.IsDir() ) {
				/* Found a dir under the mouse. Add it to the destination path */
				zDestPath += "/";
				zDestPath += pcData->m_cNode.m_zName;
				break;
			}
		}
	}
	printf( "FTPRemoteView: File dropped onto %s\n", zDestPath.c_str() );
	
	/* Start transferring the files */
	String zSource;
	String zDest;
	for( int i = 0; pcData->FindString( "file/path", &zSource, i ) == 0; i++ ) {
		Path cPath( zSource );
		zDest = zDestPath;
		zDest += "/";
		zDest += cPath.GetLeaf();
		printf( " Drag & drop %s to %s\n", zSource.c_str(), zDest.c_str() );
		
		/* Ask the Server object to start the transfer */
		m_pcServer->SendLocal( zSource, zDest );
	}
}

void FTPRemoteView::DragSelection( Point cStartPoint )
{
	/* Most of this is adapted from IconDirectoryView::DragSelection */
	
	if( m_pcServer == NULL ) { printf( "DragSelection with m_pcServer== NULL\n" ); return; }
	
	String zBase = m_pcServer->GetServerAddress();
	Message cMsg( 1234 );	/* Message code isn't important */
	
	/* Add all selected icons to the message, and find the icon under the mouse */
	int nCount = 0;
	int nLastSelected = -1;
	Point cIconPos = Point( 0,0 );
	printf( "FTPRemoteView::DragSelection(): starting to drag\n" );
	for( uint i = 0; i < GetIconCount(); i++ ) {
		if( GetIconSelected( i ) ) {
			RemoteIconData* pcData = (RemoteIconData*)GetIconData( i );
			printf( "   %s\n", pcData->m_cNode.m_zPath.c_str() );
			cMsg.AddString( "remote-file/path", pcData->m_cNode.m_zPath );
			cMsg.AddString( "server", zBase );
			
			if( Rect( GetIconPosition( i ), GetIconPosition( i ) + GetIconSize() ).DoIntersect( cStartPoint ) ) {
				cIconPos = GetIconPosition( i );
			}
			
			nLastSelected = i;
			nCount++;
		}
	}
	if( nCount == 0 ) return;
	
	/* Create a drag&drop icon */
	Bitmap cBitmap( (int)GetIconSize().x + 1, (int)GetIconSize().y + 1, CS_RGB32, Bitmap::ACCEPT_VIEWS | Bitmap::SHARE_FRAMEBUFFER );
	View* pcView = new View( Rect( Point(0,0), GetIconSize() ), "temp" );
	cBitmap.AddChild( pcView );
	
	Image* pcIcon = NULL;
	String zLabel;
	if( nCount == 1 ) {
		/* Only one file selected; use its icon */
		zLabel = GetIconString( nLastSelected, 0 );
		pcIcon = GetIconImage( nLastSelected );
	} else {
		/* Multiple files selected; use dir icon */
		zLabel.Format( "%i items", nCount );	/* TODO: localise the string */
		/* TODO: Fall back to resource if file isn't present */
		File* pcFile = new File( "/system/icons/folder.png" );
		BitmapImage* pcBitmapImage = new BitmapImage( pcFile );
		Point cSize = (GetView() == VIEW_LIST || GetView() == VIEW_DETAILS ? Point( 24,24 ) : Point( 48,48 ));
		if( pcBitmapImage->GetSize() != cSize ) pcBitmapImage->SetSize( cSize );
		pcIcon = pcBitmapImage;
	}
	if( pcIcon ) {
		RenderIcon( zLabel, pcIcon, pcView, Point(0,0) );
	}
	cBitmap.Sync();
	if( nCount != 1 ) delete( pcIcon );
	
	BeginDrag( &cMsg, cStartPoint - cIconPos, &cBitmap );
}

