#ifndef __SERVER_H__
#define __SERVER_H__

#include <util/string.h>
#include <util/handler.h>
#include <util/locker.h>
#include <map>

#include "transferthread.h"
#include "queuenode.h"


using namespace os;

class Server
{
public:
	Server( const String& zURL, Handler* pcTarget, int nParallelConnections = 2 );
	virtual ~Server();	/* I want this to be private but can't be bothered debugging the error messages yet */
	int Close();	/* Stop all transfers, close the server connection, free curl handles, stop transfer thread, delete self */
	
	int Lock();
	int Unlock();
	
	String GetServerAddress();		/* Returns the address part of the url, eg ftp://my.ftp.com   */
	
	int GetDirListing( const String& zPath, Handler* pcTarger );	/* Download a dir listing for the given path, parse it into RemoteNodes and send it to pcTarget */
	int GetRemote( const String& zRemoteSource, const String& zLocalDest );	/* Add a download to the queue */
	int SendLocal( const String& zLocalSource, const String& zRemoteDest );	/* Add an upload to the queue */

	virtual int ParseDirListing( const String& zDir, char* pzBuf, size_t nSize, Handler* pcTarget, bool bInitial );	/* Parse the text of a dir listing into RemoteNodes and send it to the RemoteView */	

	/* Methods for controlling active transfers, eg pause, cancel, resume */
	int PauseTransfer( int nNodeID );
	int ResumeTransfer( int nNodeID );
	int CancelTransfer( int nNodeID );
	
	/* Methods for manipulating the queue (ie raise/lower priority of a queued transfer) */
	/* TODO */
	
private:
	/* These methods are only called from TransferThread */

	virtual CURLcode _SetupCurlHandle( CURL* pHandle );	/* Add our settings to the curl handle */

private:
	String m_zURL;				/* Address of the server */
	
	Handler* m_pcTarget;		/* The window that created us */
	
	TransferThread* m_pcThread;	/* Our transfer thread, that does all the work */

	int m_nParallelConnections;	/* Number of concurrent connections (ie curl handles) to use */
	
	
	std::map< int, QueueNode* > m_apcQueue;	/* List of all queued and running transfers. Stored as a map from id numbers to QueueNode objects. */
	int m_nNextID;
	Locker* m_pcLock;
	/* The reason for using a std::map and giving each node an id number is so that the gui can refer to queue nodes
	   by their id rather than using a pointer. Since we can't be sure that the transfer thread hasn't deleted a node
	   while the gui is still using the pointer.
	 */
	
	friend class TransferThread;
};

class FTPServer : public Server
{
public:
	FTPServer( const String& zURL, Handler* pcTarget, int nParallelConnections );

//	int ParseDirListing( const String& zDir, char* pzBuf, size_t nSize, Handler* pcTarget, bool bInitial );
	
private:
	~FTPServer();
//	CURLcode _SetupCurlHandle( CURL* pHandle );
};



#endif	/* __SERVER_H__ */
